@@ -1,6 +1,8 @@
 """ Services module
 """
 
+from time import time
+import json
 import logging
 
 from aiohttp import ClientSession
@@ -15,12 +17,14 @@ from styler_rest_framework.exceptions.services import (
     UnexpectedError,
     ConflictError,
 )
+from styler_rest_framework.helpers.logme import logme
+from styler_rest_framework.config import defaults
 
 
 class HTTPHandler:
     """Handles HTTP operations"""
 
-    def __init__(self, identity=None, retry_on=None, headers=None):
+    def __init__(self, identity=None, retry_on=None, headers=None, log=None):
         if retry_on is None:
             self.retry_on = [503]
         else:
@@ -28,6 +32,7 @@ class HTTPHandler:
         self.headers = {}
         if headers is None:
             headers = {}
+        self.identity = None
         if identity:
             self.identity = identity
             self.headers = {
@@ -39,20 +44,30 @@ class HTTPHandler:
             **self.headers,
             **headers
         }
+        self.log_requests = log if log is not None else defaults.INTERNAL_REQUESTS_LOG_ALL
 
     async def post(self, url, params, error_handlers=None, retry=3, **kwargs):
         retry_on = kwargs.get("retry_on") or self.retry_on
         headers = self._prepare_headers(kwargs.get("headers", {}))
+        log_params = [
+            defaults.SERVICE_NAME,
+            url,
+            "POST",
+            headers.get("Authorization"),
+            json.dumps(params),
+            json.dumps(self.identity.trace_header()) if self.identity else ""
+        ]
+
         try:
             async with ClientSession() as session:
                 async with session.post(
                     url, headers=headers, json=params, ssl=True
                 ) as resp:
+                    log_params.extend([resp.status, await resp.text()])
                     if resp.status not in [200, 201]:
                         if error_handlers and resp.status in error_handlers:
                             await error_handlers[resp.status](resp)
                         await self._handle_http_errors(resp)
-
                     return await resp.json()
         except ServiceError as ex:
             # Retry
@@ -62,16 +77,30 @@ class HTTPHandler:
                 )
             else:
                 raise
+        finally:
+            if self.log_requests:
+                self.log_request(
+                    *log_params
+                )
 
     async def patch(self, url, params, error_handlers=None, retry=3, **kwargs):
         retry_on = kwargs.get("retry_on") or self.retry_on
         headers = self._prepare_headers(kwargs.get("headers", {}))
+        log_params = [
+            defaults.SERVICE_NAME,
+            url,
+            "PATCH",
+            headers.get("Authorization"),
+            json.dumps(params),
+            json.dumps(self.identity.trace_header()) if self.identity else ""
+        ]
 
         try:
             async with ClientSession() as session:
                 async with session.patch(
                     url, headers=headers, json=params, ssl=True
                 ) as resp:
+                    log_params.extend([resp.status, await resp.text()])
                     if resp.status not in [200]:
                         if error_handlers and resp.status in error_handlers:
                             await error_handlers[resp.status](resp)
@@ -86,15 +115,30 @@ class HTTPHandler:
                 )
             else:
                 raise
+        finally:
+            if self.log_requests:
+                self.log_request(
+                    *log_params
+                )
 
     async def put(self, url, params, error_handlers=None, retry=3, **kwargs):
         retry_on = kwargs.get("retry_on") or self.retry_on
         headers = self._prepare_headers(kwargs.get("headers", {}))
+        log_params = [
+            defaults.SERVICE_NAME,
+            url,
+            "PUT",
+            headers.get("Authorization"),
+            json.dumps(params),
+            json.dumps(self.identity.trace_header()) if self.identity else ""
+        ]
+
         try:
             async with ClientSession() as session:
                 async with session.put(
                     url, headers=headers, json=params, ssl=True
                 ) as resp:
+                    log_params.extend([resp.status, await resp.text()])
                     if resp.status not in [200]:
                         if error_handlers and resp.status in error_handlers:
                             await error_handlers[resp.status](resp)
@@ -109,13 +153,28 @@ class HTTPHandler:
                 )
             else:
                 raise
+        finally:
+            if self.log_requests:
+                self.log_request(
+                    *log_params
+                )
 
     async def delete(self, url, error_handlers=None, retry=3, **kwargs):
         retry_on = kwargs.get("retry_on") or self.retry_on
         headers = self._prepare_headers(kwargs.get("headers", {}))
+        log_params = [
+            defaults.SERVICE_NAME,
+            url,
+            "DELETE",
+            headers.get("Authorization"),
+            "",
+            json.dumps(self.identity.trace_header()) if self.identity else ""
+        ]
+
         try:
             async with ClientSession() as session:
                 async with session.delete(url, headers=headers, ssl=True) as resp:
+                    log_params.extend([resp.status, await resp.text()])
                     if resp.status not in [200]:
                         if error_handlers and resp.status in error_handlers:
                             await error_handlers[resp.status](resp)
@@ -130,13 +189,28 @@ class HTTPHandler:
                 )
             else:
                 raise
+        finally:
+            if self.log_requests:
+                self.log_request(
+                    *log_params
+                )
 
     async def get(self, url, error_handlers=None, retry=3, **kwargs):
         retry_on = kwargs.get("retry_on") or self.retry_on
         headers = self._prepare_headers(kwargs.get("headers", {}))
+        log_params = [
+            defaults.SERVICE_NAME,
+            url,
+            "GET",
+            headers.get("Authorization"),
+            "",
+            json.dumps(self.identity.trace_header()) if self.identity else ""
+        ]
+
         try:
             async with ClientSession() as session:
                 async with session.get(url, headers=headers, ssl=True) as resp:
+                    log_params.extend([resp.status, await resp.text()])
                     if resp.status != 200:
                         if error_handlers and resp.status in error_handlers:
                             await error_handlers[resp.status](resp)
@@ -151,6 +225,55 @@ class HTTPHandler:
                 )
             else:
                 raise
+        finally:
+            if self.log_requests:
+                self.log_request(
+                    *log_params
+                )
+
+    def log_request(
+            self,
+            origin_service: str = None,
+            path: str = None,
+            method: str = None,
+            auth: str = None,
+            request_body: str = None,
+            request_tags: str = None,
+            response_status_code: int = None,
+            response_body: str = None):  # pragma: no coverage
+        """Logs the request/response
+
+        :param origin_service: Service sender, defaults to None
+        :type origin_service: str, optional
+        :param path: URL, defaults to None
+        :type path: str, optional
+        :param method: HTTP Method, defaults to None
+        :type method: str, optional
+        :param auth: Auth JWT, defaults to None
+        :type auth: str, optional
+        :param request_body: JSON string, defaults to None
+        :type request_body: str, optional
+        :param request_tags: Identifiers, defaults to None
+        :type request_tags: str, optional
+        :param response_status_code: HTTP Response status code, defaults to None
+        :type response_status_code: int, optional
+        :param response_body: JSON string, defaults to None
+        :type response_body: str, optional
+        """
+        try:
+            logme({
+                "dataset": defaults.INTERNAL_REQUESTS_DATASET,
+                "table": defaults.INTERNAL_REQUESTS_TABLE,
+                "rows": [
+                    (origin_service, path, method, auth, request_body, int(time()), request_tags, response_status_code, response_body)
+                ]
+            })
+        except Exception as ex:
+            logging.warning(f"Could not log request: {str(ex)}")
+
+    def get_trace_header(self, headers=None):
+        headers = headers or self.headers
+
 
     def _prepare_headers(self, overrides):
         headers = self.headers
